###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

VERSION = "master"

import os, tarfile
from obt import dep, host, path, cmake, git, make, pathtools
from obt.deco import Deco
from obt.wget import wget
from obt.command import Command

deco = Deco()

###############################################################################

class vpf(dep.Provider):

  def __init__(self): ############################################
    super().__init__("vpf")
    #print(options)
    self.source_root = path.builds()/"vpf"
    self.build_dest = path.builds()/"vpf"/".build"
    self.sdk_dir = path.Path("/opt/nvencsdk")
    self._archlist = ["x86_64"]

  def __str__(self): ##########################################################

    return "VPF (github-%s)" % VERSION

  def wipe(self): #############################################################
    os.system("rm -rf %s"%self.source_root)

  def build(self): ##########################################################

    if not self.sdk_dir.exists():
        print(deco.red("you need the nvenc SDK at <%s>" % self.sdk_dir ))
        print(deco.red("download @ https://developer.nvidia.com/nvidia-video-codec-sdk"))
        print(deco.yellow(" * requires login which is why I cannot download it for you!"))
        assert(False)

    python = dep.require("python")
    pybind11 = dep.require("pybind11")

    #########################################
    # fetch source
    #########################################

    if not self.source_root.exists():
        git.Clone("https://github.com/tweakoz/VideoProcessingFramework",self.source_root,VERSION)

    #########################################
    # prep for build
    #########################################

    ok2build = True
    if self.should_incremental_build:
        os.chdir(self.build_dest)
    else:

        pathtools.mkdir(self.build_dest,clean=True)
        os.chdir(self.build_dest)

        cmakeEnv = {
            "CMAKE_BUILD_TYPE": "RELEASE",
            "VIDEO_CODEC_SDK_INCLUDE_DIR": self.sdk_dir/"include",
            "GENERATE_PYTHON_BINDINGS": True
        }

        cmake_ctx = cmake.context("..",env=cmakeEnv)
        ok2build = cmake_ctx.exec()==0

    #########################################
    # build
    #########################################

    OK = True
    if ok2build:
        OK = (make.exec("install")==0)
    return OK

  def linkenv(self): ##########################################################
    LIBS = ["ork_vpf"]
    return {
        "LIBS": LIBS,
        "LFLAGS": ["-l%s"%item for item in LIBS]
    }



import obt._dep_enumerate
import obt._dep_node
from obt.deco import Deco 
import sys

deco = Deco()

###############################################################################
def instance(named,with_overrides=True):
  n = obt._dep_node.DepNode.nodeForName(named,with_overrides)
  if n and hasattr(n,"instance") and n.instance.supports_host:
    return n.instance
  return None
###############################################################################
def module_class(named,with_overrides=True):
  n = obt._dep_node.DepNode.nodeForName(named,with_overrides)
  if n and hasattr(n,"module_class") and n.instance.supports_host:
    return n.module_class
  return None
###############################################################################
class Chain:
  ############################
  def __init__(self,named_or_list):
    self._list = list()
    self._dict = dict()
    ##########################
    index = 0
    indexed = dict()
    def visit_provider(provider):
      assert(hasattr(provider,"_name"))
      #print(provider._name)
      nonlocal self, index
      ############################
      # new item ?
      ############################
      if provider._name not in self._dict.keys():
        self._dict[provider._name] = provider
        provider._topoindex = index
        indexed[index] = provider
        index+=1
      ############################
      # recurse into children
      ############################
      for name in provider._required_deps.keys():
        inst = provider._required_deps[name]
        if inst != None:
          visit_provider(inst)
    ##########################
    def perform_on_node(named):
      ##########################
      # check node comformity
      ##########################
      root = obt._dep_node.DepNode.nodeForName(named)
      #print(root)
      if (root==None):
        print(deco.err("DepNode<%s> does not have instance - check that the provider/class is named correctly!"%named))
        sys.exit(-1)
      if (root!=None) and (not hasattr(root,"instance")):
        print(deco.err("DepNode<%s> does not have instance - check that the provider/class is named correctly!"%named))
        sys.exit(-1)
      ##########################
      if root.instance!=None:
        visit_provider(root.instance)
    ##########################
    # start recursion at root
    if isinstance(named_or_list,str):
      perform_on_node(named_or_list)
    elif isinstance(named_or_list,list):
      for item in named_or_list:
        perform_on_node(item)
    ##########################
    # topological unsorted
    ##########################
    topo_unsorted = dict()
    for index in indexed.keys():
      topo_unsorted[index] = set()
      p = indexed[index]
      #print(p)
      deps = p._required_deps
      for d in deps.keys():
        inst = deps[d]
        if inst != None:
          topo_unsorted[index].add(inst._topoindex)
    #print(topo_unsorted)
    ##########################
    # topological sorted
    ##########################
    from toposort import toposort, toposort_flatten
    #print("sorted:")
    sorted = list(toposort_flatten(topo_unsorted))
    #print(sorted)
    for i in reversed(sorted):
      n = indexed[i]
      #print(i,n._name)
      self._list.append(n)
      self._dict[n._name]=n
      #if i._name not in self._dict.keys():
       #self._dict[i._name] = i
       #self._list.append(i)
    ##########################
  



#!/usr/bin/env python3 

import os, ci_common, argparse
from obt import path, deco
deco = deco.Deco()

os.environ["DOCKER_BUILDKIT"]="1"

parser = argparse.ArgumentParser(description='obt.build CICD master image builder')
parser.add_argument('--secretsdir', help='ssh key path' )

_args = vars(parser.parse_args())

have_secrets = False
if "SECRETSDIR" in os.environ:
  secrets_dir = path.Path(os.environ["SECRETSDIR"])
  have_secrets = True
if "secretsdir" in _args and _args["secretsdir"]!=None:
  secrets_dir = path.Path(_args["secretsdir"])
  have_secrets = True

if not have_secrets:
  print(deco.err("##########################################################"))
  print()
  print(deco.err(parser.format_usage()))
  print(deco.err("##########################################################"))
  assert(False)


assert(secrets_dir/"buildkey.rsa.pub").exists()

cmdlist = list()
cmdlist += ["docker","build"]
cmdlist += ["-f", "master.dockerfile"]
cmdlist += ["-t", "obtcicd/master_focal"]
cmdlist += ["--secret", "id=ssh_public,src=%s/buildkey.rsa.pub"%secrets_dir]
cmdlist += ["--secret", "id=ssh_private,src=%s/buildkey.rsa"%secrets_dir]
cmdlist += ["."]

ci_common.sync_subprocess(cmdlist)




###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

VERSION = "1.2.198.1"
MD5 = "c26c3febbf14faef595d7c26715a5472"

import os, tarfile
from obt import dep, host, path, cmake, git, make, command, wget, env, log, pathtools
from obt.deco import Deco
from obt.wget import wget
from obt.command import Command

deco = Deco()

###############################################################################

class _vulkan_from_lunarg(dep.Provider):

  def __init__(self): ############################################
    super().__init__("vulkan")
    #print(options)
    self.fullver = VERSION
    self.source_root = path.builds()/"vulkan"
    self.build_dest = path.builds()/"vulkan"/".build"
    #self._archlist = ["x86_64"]
    self._oslist = ["Linux"]
    if host.IsX86_64:
      self.sdk_dir = self.source_root/VERSION/"x86_64"
    elif host.IsAARCH64:
      self.sdk_dir = self.source_root/VERSION/"aarch64"

  def __str__(self): ##########################################################

    return "Vulkan (lunarg-%s)" % VERSION

  ########################################################################
  @property
  def download_name(self):
    if host.IsX86_64:
      nam = "vulkansdk-linux-x86_64-%s.tar.gz"%VERSION
    elif host.IsX86_32:
      nam = "vulkansdk-linux-i386-%s.tar.gz"%VERSION
    elif host.IsAARCH64:
      nam = "vulkansdk-linux-aarch64-%s.tar.gz"%VERSION
    return nam 
  
  @property
  def download_URL(self):
    return "https://sdk.lunarg.com/sdk/download/%s/linux/%s"%(VERSION,self.download_name)

  @property
  def download_MD5(self):
    return MD5

  ########################################################################
  @property
  def revision(self):
    return VERSION
  #######################################################################

  def env_init(self):
    if self.sdk_dir.exists():
      log.marker("registering Vulkan(%s) SDK"%VERSION)
      env.prepend("LD_LIBRARY_PATH",self.sdk_dir/"lib")
      env.append("PATH",self.sdk_dir/"bin")
      env.set("VULKAN_SDK",self.sdk_dir) # for cmake
      env.set("VK_LAYER_PATH", self.sdk_dir/"etc"/"vulkan"/"explicit_layer.d")
      env.set("OBT_VULKAN_VERSION",VERSION) # for OBT internal
      env.set("OBT_VULKAN_ROOT",self.sdk_dir) # for OBT internal

  def areRequiredSourceFilesPresent(self):
    return (self.sdk_dir/".."/"setup-env.sh").exists()
  def areRequiredBinaryFilesPresent(self):
    return (self.sdk_dir/"bin"/"vkconfig").exists()

  def build(self): ##########################################################

    url = self.download_URL
    ok = wget(urls=[url],output_name=self.download_name,md5val=MD5)

    print(ok)
    if not ok:
      return False

    self.source_root.mkdir(parents=True,exist_ok=True)
    os.chdir(self.source_root)
    ok = (command.system(["rm","-rf",VERSION])==0)
    if not ok:
      return False
    ok = (command.system(["tar","xvf",path.downloads()/self.download_name])==0)
    if not ok:
      return False

    #samples_build_dir = self.sdk_dir/".."/"samples"/".build-samples"
    #pathtools.mkdir(samples_build_dir,clean=True)
    #samples_build_dir.chdir()
    #ok = (command.system(["cmake",".."])==0)
    #if not ok:
    #  return False
    #ok = (command.system(["make","-j",host.NumCores])==0)
    #if not ok:
    #  return False

    return True

###############################################################################

class _vulkan_from_system(dep.StdProvider):
  name = "vulkan"
  def __init__(self):
    super().__init__(_vulkan_from_system.name)
    self.fullver = "1.2.131"
    self._builder = dep.NopBuilder(_vulkan_from_system.name)
  ########################################################################
  @property
  def _fetcher(self):
    return dep.NopFetcher(_vulkan_from_system.name)
  #######################################################################
  ########
  def __str__(self):
    return "vulkan"
  def env_init(self):
    log.marker("registering Vulkan(%s) SDK"%self.fullver)
    env.set("VULKAN_VER",self.fullver)
  def install_dir(self):
    return path.Path("/usr")

###############################################################################

if host.IsAARCH64:
  BASE = _vulkan_from_system
elif host.IsX86_64 or host.IsX86_32:
  BASE = _vulkan_from_lunarg
else:
  assert(False)

###############################################################################

class vulkan(BASE):
  def __init__(self):
    super().__init__()
  ########
  @property
  def include_dir(self):
    return path.include_dir()



from obt import dep, pip, path, dep, host
from obt.command import Command
###############################################################################

class pydefaults(dep.Provider):

  def __init__(self): ############################################
    super().__init__("pydefaults")
    build_dest = path.builds()/"pydefaults"
    self.build_dest = build_dest
    self.python = self.declareDep("python")

  def build(self): ############################################################
    #PYTHON = dep.instance("python")
    modules = [ "numpy",
                "scipy",
                "numba",
                "pyopencl",
                "matplotlib",
                "pyzmq",
                "mido",
                "ipython",
                "traitlets",
                "imgui_bundle",
                "PyGLM",
                "opencv-contrib-python",
                "solidpython2",
                "manifold3d", # TODO: disable on aarch64/linux
                "scikit-image",
                "MDAnalysis",
                "MDAnalysisData",
                #"cadquery[ipython]" # does not work on macos/arm64
              ]
    modules += ["Pillow","jupyter","plotly","trimesh","asciidoc"]
    if host.IsDarwin == False:
      modules += ["pysqlite3","pyudev"]
    #################

    ret = Command([self.python.executable,"-m","pip","install","--upgrade"]+modules).exec()

    print("pydefaults build ret<%d>"%int(ret))
    return (ret==0)

  def areRequiredSourceFilesPresent(self):
    return (self.python.site_packages_dir/"numpy"/"_globals.py").exists()

  def areRequiredBinaryFilesPresent(self):
    return self.areRequiredSourceFilesPresent()

###############################################################################
